/*
 * Batch.h : functions that operate on batch data for use in our algorithm.
 * 	     These are the two consolidation methods used after the stream has completed.
 */


#ifndef __BATCH_H
#define __BATCH_H

struct Facility;

/*
 * Returns an array of integers of size k
 * These are each distinct and in the range [0, n-1]
 * These represent k initial means for a k-means solution
 * 	as per the ORSS distribution as specified in
 * 		"The Effectiveness of Lloyd-type methods"
 * 		by Rafail Ostrovsky, Yuval Rabani, Leonard Schulman, and Chaitanya Swamy 
 *		In FOCS 2006
 */
int * ORSSSeed(Facility * fs[], int n, int k);


/*
 * Performs one-swap local search for k-means on the given set of facilities.
 * The starting point for the local search is in start[].
 * 		{ fs[start[0]], fs[start[1]], ... fs[start[k-1]] }
 * The ending point is returned via the start[] parameter.
 */
void LocalSearch(Facility * fs[], int n, int k, int start[]);


#endif
