#include "point.h"

using namespace std;

double readDouble(FILE* file);
int readInt(FILE * file);

void init(Point *p, int d)
{
	p->coordinates = (float*) calloc(d, sizeof(float)); 
	p->dimension = d;
	p->weight = 1;
}

void freePoint(Point *p)
{
	free(p->coordinates);
}


bool equals(Point * a, Point * b)
{
	int i;
	for (i=0; i < a->dimension; i++)
	{
		if( a->coordinates[i] != b->coordinates[i] )
		{
			return false;
		}
	}
	return true;
}

double distanceSQ(Point * a, Point * b)
{
	double sum = 0.0;
	int i;
	double it;
	for (i=0; i < a->dimension; i++)
	{
		it = a->coordinates[ i ] - b->coordinates[i];
		it = it * it;
		sum += it;
	}
	
	return sum;	
}

/* performs a DEEP copy of a = b
 * Assumes dimension is equal and already set
 *	and that coordinates is already initialized 
 * It also copies the weight.
 */
void copy(Point * a, Point * b)
{
	int i;
	for (i=0; i < a->dimension; i++)
	{
		a->coordinates[i] = b->coordinates[i];
	}
	a->weight = b->weight;
}


/*
 *  Returns the next point of a file(Liadan format from StreamKM++)
 * 	(function from their Point.c file)
 * */
void getNextPoint(FILE* file, Point * p, int dim)
{
	// skip first number
	float nextNumber;
	nextNumber = readDouble(file);
	for(int l = 0; l < p->dimension; l++)
	{
		nextNumber = readDouble(file);
		p->coordinates[l] = nextNumber;
	}	
	p->weight = 1.0;
}


void getNextWeightedPoint(FILE *file, Point *p, int dim)
{
	float nextNumber;
	int nextInt;
	nextInt = readInt(file);
	p->weight = nextInt;
	for(int l = 0; l < p->dimension; l++)
	{
		nextNumber = readDouble(file);
		p->coordinates[l] = nextNumber;
	}	
}




/*
 * Reads a double. Ignores leading spaces.
 * 	(function from Ackermann et al's Point.c file)
 * (modified: c != ',')
 */
double readDouble(FILE* file)
{
	
	char c;
	static char buf[100];
	
	int i = 1;
	while((c = fgetc(file)) ==' ' && c != EOF){}
	
	buf[0] = c;
	    
	while((c = fgetc(file)) != '\n' && c != ' ' && c != EOF && c != ',' ){
		buf[i++] = c;
	}
	buf[i]=0;
	
	return atof(buf);
}


int readInt(FILE* file)
{
	char c;
	static char buf[100];
	int i = 1;
	while((c = fgetc(file)) ==' ' && c != EOF){ }
	buf[0] = c;
	   
	while((c = fgetc(file)) != '\n' && c != ' ' && c != EOF && c != ',' ){
		buf[i++] = c;
	}
	
	buf[i]=0;
	
	return atoi(buf);
}


double distSQToClosest(Point * p, Point * ctrs, int numCtrs)
{
		return distSQToClosest(p, ctrs, 0, numCtrs);
}


double distSQToClosest(Point * p, Point * ctrs, int s, int e)
{
	double closest = distanceSQ( p, ctrs + s);
	double dis;
	int i;
	for (i = s+1; i < e; i++)
	{
		dis = distanceSQ(p, ctrs+i);
		if ( dis < closest )
			closest = dis;
	}
	return closest;
}

/*
 * Deletes coords of each point in a. 
 */
void doDelete(Point a[], int n)
{
	int i;
	for (i=0; i < n; i++)
	{
		free( a[i].coordinates );
	}
}


int findClosest(Point * data, int n, Point * a)
{
	int indexOfClosest = 0;
	double distToClosest = distanceSQ(a, data+0);
	int i;
	double dis;
	for(i=1; i < n; i++)
	{
		dis = distanceSQ(a, data+i);
		if ( dis < distToClosest)
		{
			distToClosest = dis;
			indexOfClosest = i;
		}
	}
	return indexOfClosest;
}


void initArray(Point * ps, int n, int d)
{
	int i;
	for(i=0; i < n; i++)
	{
		init(ps + i, d);
	}
}

void copyArray(Point * a, Point * b, int n)
{
	int i;
	for (i=0; i < n; i++)
	{
		copy(a+i, b+i);
	}
}

bool contains(Point * set, int size, Point * p)
{
	int i;
	for (i=0; i < size; i++)
	{
		if ( equals(set + i, p) )
			return true;
	}
	return false;
}


void outputWeightedPoint(Point * p, FILE * out)
{
	fprintf(out, "%d", p->weight);
	int i;
	for (i=0; i < p->dimension; i++)
	{
		fprintf(out, ",%f", p->coordinates[i]);
	}
	fprintf(out, "\n");
}
