/*
 * A structure to represent a Point in Streaming k-means.
 *
 *
 */

#ifndef __POINT_H
#define __POINT_H


#include <stdio.h>
#include <stdlib.h>


struct Point
{
	float * coordinates;
	int dimension;
	int weight;
};

/*
 * Initializes an empty point having d dimensions and a weight of one.
 */
void init(Point * p, int d);

/*
 * Releases the memory associated with this point.
 */
void freePoint(Point * p);

/*
 * Two points are considered equal if they have each coordinate as equal.
 * Note that in this application, all points will have the same dimensionality
 * and thus checking for this is unnecessary.
 *
 * Note that this _doesn't_ check the weights to be equal.
 */
bool equals(Point * a, Point * b);

/*
 * Returns the square of the distance between these two points.
 */
double distanceSQ(Point * a, Point * b);

/*
 * Reads the next point from this file. 
 * This is the Liadan format from StreamKM++
 * (The function is adapted from their code)
 */
void getNextPoint(FILE* file, Point * p, int dim);  

/*
 * Reads the next point from this file. 
 * This is the Liadan format from StreamKM++
 * (The function is adapted from their code)
 * 
 * For the weighted points, the first item in the line is interpreted
 * as the point's weight, rather than skipped.
 */
void getNextWeightedPoint(FILE *file, Point *p, int dim);

/*
 * Determines if the given set contains the designated Point.
 */
bool contains(Point * set, int size, Point * p);

/*
 * These two points find the nearest center to the given Point.
 * 
 * The first looks at each center and finds the closest of those.
 * The second looks only in the given range.
 */
double distSQToClosest(Point * p, Point * ctrs, int numCtrs);

double distSQToClosest(Point * p, Point * ctrs, int start, int end);

/*
 * Performs a deep copy of a <-- b
 */
void copy(Point * a, Point * b);

/*
 * Deletes coords of each point in a.
 */
void doDelete(Point a[], int n);

/*
 * Returns the index of the nearest of the first n points in a
 * to the given Point data.
 */
int findClosest(Point * data, int n, Point * a);

/*
 * Initializes an array of n points, each of dimension d.
 */
void initArray(Point * ps, int n, int d);

/*
 * Copies the first n elements of array b to array a.
 */
void copyArray(Point * a, Point * b, int n);

/*
 * Writes the given point, complete with weight, to the designated file.
 */
void outputWeightedPoint(Point * p, FILE * out); 

#endif
